<?php

namespace FiloBlu\OrderAdditionalData\Controller\Adminhtml\Order;

use Magento\Backend\App\Action;
use Magento\Framework\Controller\Result\JsonFactory;
use Magento\Framework\Controller\ResultInterface;
use Magento\Framework\DataObject;
use Magento\Framework\DataObjectFactory;
use Magento\Framework\Module\Manager;
use Magento\Sales\Api\Data\OrderInterface;
use Magento\Sales\Api\OrderRepositoryInterface;
use Magento\Sales\Model\Order;

/**
 * Class Save
 * @package FiloBlu\OrderAdditionalData\Controller\Order
 */
class Save extends Action
{
    /**
     * @var JsonFactory
     */
    private $jsonFactory;

    /**
     * @var OrderRepositoryInterface
     */
    private $orderRepository;

    /**
     * @var DataObjectFactory
     */
    private $dataObjectFactory;
    /**
     * @var Manager
     */
    private $moduleManager;

    /**
     * Save constructor.
     * @param Action\Context $context
     * @param OrderRepositoryInterface $orderRepository
     * @param DataObjectFactory $dataObjectFactory
     * @param Manager $moduleManager
     * @param JsonFactory $jsonFactory
     */
    public function __construct(
        Action\Context           $context,
        OrderRepositoryInterface $orderRepository,
        DataObjectFactory        $dataObjectFactory,
        Manager                  $moduleManager,
        JsonFactory              $jsonFactory
    )
    {
        parent::__construct($context);
        $this->jsonFactory = $jsonFactory;
        $this->orderRepository = $orderRepository;
        $this->dataObjectFactory = $dataObjectFactory;
        $this->moduleManager = $moduleManager;
    }

    /**
     * @return ResultInterface
     */
    public function execute()
    {
        $orderId = $this->getRequest()->getParam('id');
        $section = $this->getRequest()->getParam('filoblu_additional_data_section');

        if ($orderId) {
            $order = $this->orderRepository->get($orderId);
            $this->storeData($order, $this->getData(), $section);
            $this->orderRepository->save($order);
        }

        return $this->jsonFactory->create()->setData([]);
    }

    /**
     * @param Order|OrderInterface $order
     * @param DataObject $data
     * @param $section
     * @return OrderInterface|Order
     */
    public function storeData($order, DataObject $data, $section)
    {
        foreach ($this->getModuleMapping() as $module => $map) {

            if (!$this->moduleManager->isEnabled($module)) {
                continue;
            }

            if ($map['section'] !== $section) {
                continue;
            }

            $setter = $map['setter'];

            foreach ($setter as $from => $to) {

                $value = $data->getDataByPath($from);
                if (is_callable($to)) {
                    $to($value, $order);
                    continue;
                }
                $order->setData($to, $value);
            }

        }

        return $order;
    }

    /**
     * @return array
     */
    public function getModuleMapping(): array
    {
        return [
            'FiloBlu_OrderAdditionalData' => [
                'section' => 'general',
                'setter'  => [
                    'order/pay_exchangerate'                   => 'pay_exchangerate',
                    'order/fbnetsuite_theoric_shipping_amount' => 'fbnetsuite_theoric_shipping_amount'
                ]
            ],
            'Ebizmarts_MailChimp'         => [
                'section' => 'mailchimp',
                'setter'  => [
                    'order/mailchimp_campaign_id'        => 'mailchimp_campaign_id',
                    'order/mailchimp_abandonedcart_flag' => 'mailchimp_abandonedcart_flag',
                    'order/mailchimp_landing_page'       => 'mailchimp_landing_page',
                    'order/mailchimp_flag'               => 'mailchimp_flag'
                ]
            ],
            'FiloBlu_CheckoutExtend'      => [
                'section' => 'checkout',
                'setter'  => [
                    'order/checkout_invoice'               => 'checkout_invoice',
                    'order/checkout_invoice_vat_number'    => 'checkout_invoice_vat_number',
                    'order/checkout_invoice_fiscal_code'   => 'checkout_invoice_fiscal_code',
                    'order/checkout_invoice_sdi'           => 'checkout_invoice_sdi',
                    'order/checkout_invoice_pec'           => 'checkout_invoice_pec',
                    'order/checkout_invoice_business_name' => 'checkout_invoice_business_name',
                    'order/dni'                            => 'dni'
                ]
            ],
            'FiloBlu_Storelocator'        => [

                'section' => 'pickup_in_store',
                'setter'  => [
                    'order/pickup_in_store_id'                    => 'pickup_in_store_id',
                    'order/pickup_in_store_email'                 => 'pickup_in_store_email',
                    'order/pickup_in_store_confirm_date'          => 'pickup_in_store_confirm_date',
                    'order/pickup_in_store_hash'                  => 'pickup_in_store_hash',
                    'order/pickup_in_store_confirm_received_date' => 'pickup_in_store_confirm_received_date',
                    'order/pickup_in_store_email_send_date'       => 'pickup_in_store_email_send_date'
                ]
            ],
            'FiloBlu_RefiloPayments'      => [
                'section' => 'fiscal_information',
                'setter'  => [
                    'order/extension_attributes/fiscal_information/request_invoice'                   => function ($value, $order) {
                        $order->getExtensionAttributes()->getFiscalInformation()->setRequestInvoice($value);
                    },
                    'order/extension_attributes/fiscal_information/country_code'                   => function ($value, $order) {
                        $order->getExtensionAttributes()->getFiscalInformation()->setCountryCode($value);
                    },
                    'order/extension_attributes/fiscal_information/national_identification_number' => function ($value, $order) {
                        $order->getExtensionAttributes()->getFiscalInformation()->setNationalIdentificationNumber($value);
                    },
                    'order/extension_attributes/fiscal_information/vat_number'                     => function ($value, $order) {
                        $order->getExtensionAttributes()->getFiscalInformation()->setVatNumber($value);
                    },
                    'order/extension_attributes/fiscal_information/destination_code'               => function ($value, $order) {
                        $order->getExtensionAttributes()->getFiscalInformation()->setDestinationCode($value);
                    },
                    'order/extension_attributes/fiscal_information/company_name'                   => function ($value, $order) {
                        $order->getExtensionAttributes()->getFiscalInformation()->setCompanyName($value);
                    },
                    'order/extension_attributes/fiscal_information/pec_email'                      => function ($value, $order) {
                        $order->getExtensionAttributes()->getFiscalInformation()->setPecEmail($value);
                    },
                    'order/extension_attributes/fiscal_information/dob'                            => function ($value, $order) {
                        $order->getExtensionAttributes()->getFiscalInformation()->setDob($value);
                    }
                ]]
        ];
    }

    /**
     * @return DataObject
     */
    public function getData(): DataObject
    {
        $data = $this->getRequest()->getParam('filoblu_additional_data', []);

        $output = [];
        foreach ($data as $key => $value) {

            $paths = explode('/', $key);

            $current = &$output;
            foreach ($paths as $path) {
                if (!isset($current[$path])) {
                    $current[$path] = [];
                }

                $current = &$current[$path];
            }

            $current = $value;
            unset($current);
        }

        return $this->dataObjectFactory->create(['data' => $output]);
    }
}
